#' summary.freqlist
#' 
#' Summarize the \code{freqlist} object
#'
#' @param object an object of class \code{\link{freqlist}}
#' @param single a logical value indicating whether to collapse results created using a groupBy variable into a single table for printing
#' @param labelTranslations A character vector giving the labels. Overrides the labels in `freqlist`.
#' @param ... additional arguments passed to the \code{\link[knitr]{kable}} function
#' @return Invisibly returns \code{object}, and uses \code{\link[knitr]{kable}} to print the object.
#' @seealso \code{\link[base]{table}}, \code{\link[stats]{xtabs}}, \code{\link[knitr]{kable}}
#'
#' @examples
#' # load mockstudy data
#' data(mockstudy)
#' tab.ex <- table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA = "ifany")
#' noby <- freqlist(tab.ex, na.options = "include")
#' summary(noby)
#' withby <- freqlist(tab.ex, groupBy = c("arm","sex"), na.options = "showexclude")
#' summary(withby)
#' @author Tina Gunderson
#' @export
#' 
summary.freqlist <- function(object, single = FALSE, labelTranslations = NULL, ...){
  #require(knitr, quietly = TRUE)
  if (!is.logical(single)) stop("single must be TRUE or FALSE")
  if(!is.null(labelTranslations)) labels(object) <- labelTranslations
  
  # fmtdups <- function(vec){
  #   x <- vec
  #   x[is.na(x)] <- "NA"
  #   y <- c(NA, x[-1])
  #   x[x==y] <- ""
  #   return(x)
  # }
  
  ## changed on 11/18/16 by EPH. The other one wasn't working in cases like the second testthat example (bug reported by Emily Lundt)
  fmtdups <- function(tab)
  {
    tab <- as.matrix(tab)
    tab[is.na(tab)] <- "NA"
    output <- tab
    num <- max(stringr::str_count(tab, ","))
    
    for(col in 1:ncol(tab))
    {
      tmp <- apply(tab[, 1:col, drop = FALSE], 1, paste, collapse = paste0(rep(",", num + 1), collapse = "")) # in R >= 3.3.0, we could use strrep instead
      output[duplicated(tmp), col] <- ""
    }
    output
  }
  
  if(is.null(object[["labels"]])){
    cnames <- names(object[["freqlist"]])
  } else {
    cnames <- c(object[["labels"]], "Freq", "cumFreq", "freqPercent","cumPercent")
  }
  if(is.null(object[["byVar"]]) || single){
    freqdf <- object[["freqlist"]]
    if(ncol(freqdf)>5){
      # freqdf[, 1:(ncol(freqdf)-5)] <- apply(freqdf[, 1:(ncol(freqdf)-5), drop = FALSE], 2, fmtdups)
      freqdf[, 1:(ncol(freqdf)-4)] <- fmtdups(freqdf[, 1:(ncol(freqdf)-4), drop = FALSE])
    }
    print(knitr::kable(freqdf, row.names = FALSE, col.names = cnames, ...))
  } else {
    byVar <- object[["byVar"]]
    freqdf <- object[["freqlist"]]
    for(i in match(byVar, names(freqdf))) {
      if(sum(is.na(freqdf[, i])) > 0) {freqdf[, i] <- addNA(freqdf[, i])}
    }
    printlist <- by(freqdf, freqdf[, rev(byVar)], FUN = data.frame)
    names(printlist) <- gsub("[.]",", ", levels(interaction(rev(freqdf[,byVar]))))
    for(i in 1:length(printlist)){
      if(!is.null(printlist[[i]])){
        if(nrow(printlist[[i]]) > 1){
          sublist <- printlist[[i]]
          # sublist[, 1:(ncol(sublist)-5)] <- apply(sublist[, 1:(ncol(sublist)-5), drop = FALSE], 2, fmtdups)
          sublist[, 1:(ncol(sublist)-4)] <- fmtdups(sublist[, 1:(ncol(sublist)-4), drop = FALSE])
          print(knitr::kable(sublist, row.names = FALSE, col.names = cnames, ...))
        } else {
          print(knitr::kable(printlist[[i]], row.names = FALSE, col.names = cnames, ...))
        }
      }
    }
  }
  invisible(object)
}